using System;
using System.Collections.Generic;
using System.Text;
using Omega.Daq;
using Omega.Daq.Daq2401;

namespace CSharpSample
{
    /// <summary>
    /// Sample code to demonstrate the use of the Omega.Daq API.  To run the hardware portion of this sample code, connect a 0 - 10 V signal to at least one of channels A1H, A2H, A3H, and A4H.
    /// </summary>
    class Program
    {
        static void Main(string[] args)
        {
            //run a test with the demo device
            Console.WriteLine("Testing demo device.");
            Device demoDevice = new Daq2401Demo("DEMO");
            RunTests(demoDevice);

            //check for real devices
            IList<Device> devices = Device.FindConnectedDevices();
            if (devices.Count > 0)
            {
                //use the first detected device for testing purposes
                RunTests(devices[0]);
            }
            else
            {
                Console.WriteLine("No devices detected.");
            }

            Console.Write("Done. Press <Enter> to exit.");
            Console.ReadLine(); //wait for Enter key
        }

        /// <summary>
        /// Run test/demonstration methods.
        /// </summary>
        /// <param name="device">The device to test.</param>
        private static void RunTests(Device device)
        {
            const int NumChannels = 4;

            Console.WriteLine("Running tests on device {0}", device.SerialNumber);
            
            //Create a Scan Group with some Channel objects to be used for the tests
            //use a ChannelConfiguration structure to easily copy the same setup to multiple channels
            ChannelConfiguration configuration = new ChannelConfiguration();
            configuration.InputRange = InputRange.VoltsPlusMinus10;
            configuration.ChannelMode = ChannelMode.SingleEndedHigh;
            //create channels - we'll be using the High input of each channel, with the range set to +/- 10V
            Channel[] scanGroup = new Channel[NumChannels]; //could also use List<Channel>, or any other ICollection<Channel>
            for (int i = 0; i < NumChannels; i++)
            {
                scanGroup[i] = new Channel(i + 1, configuration);
            }
            
            //Demonstrate EasyRead single-step command:
            EasyReadTest(device, scanGroup);
            Console.WriteLine();

            //Demonstrate EasyScan single-step command
            EasyScanTest(device, scanGroup);
            Console.WriteLine();
            
            //Demonstrate standard method
            DaqError standardTest = StandardScanTest(device, scanGroup);
            if (standardTest.IsError)
            {
                Console.WriteLine("Error - Code: {0} - Message: {1}", standardTest.Name, standardTest.Message);
                if (device.IsConnected)
                {
                    device.Disconnect();
                }
            }
            Console.WriteLine();

            Console.WriteLine();
        }

        /// <summary>
        /// Demonstrates the EasyRead single-step command. EasyRead reads one channel at a time.
        /// </summary>
        /// <param name="device">The device.</param>
        /// <param name="scanGroup">The scan group.</param>
        private static void EasyReadTest(Device device, ICollection<Channel> scanGroup)
        {
            Console.WriteLine("EasyRead single-step command:");
            
            //call EasyRead on each channel in our scan group array and print the output

            foreach (Channel chan in scanGroup)
            {
                Console.Write("Channel {0}: ", chan.Number);
                double voltage = 0;
                DaqError response = device.EasyRead(chan, out voltage);
                if (response.IsError)
                    Console.WriteLine("Error.");
                else //successful read
                    Console.WriteLine(voltage);
            }
        }

        /// <summary>
        /// Demonstrates the EasyScan single-step command. EasyScan reads multiple channels (the scan group) a specified number of times.
        /// </summary>
        /// <param name="device">The device.</param>
        /// <param name="scanGroup">The scan group.</param>
        private static void EasyScanTest(Device device, Channel[] scanGroup)
        {
            const double ScanRate = 100; //samples/second
            const int NumScans = 3;
            double[] data = new double[scanGroup.Length * NumScans]; //the data is returned in a standard one-dimensional array: first one scan, then the next, etc.

            Console.WriteLine("EasyScan single-step command:");

            //call EasyScan and print the output
            DaqError response = device.EasyScan(scanGroup, NumScans, ScanRate, out data);
            if (response.IsError)
            {
                Console.WriteLine("Error.");
            }
            else //successful scan
            {
                //iterate over the results
                for (int s = 0; s < NumScans; s++)
                {
                    Console.WriteLine("Scan #{0}", s);
                    for (int c = 0; c < scanGroup.Length; c++)
                    {
                        int currentIndex = (s * scanGroup.Length) + c;
                        Console.WriteLine("Result #{0}: {1}", c, data[currentIndex]);
                    }
                }
            }
        }

        /// <summary>
        /// Demonstrates the standard API for reading data.
        /// </summary>
        /// <param name="device">The device.</param>
        /// <param name="scanGroup">The scan group.</param>
        private static DaqError StandardScanTest(Device device, Channel[] scanGroup)
        {
            const double ScanRate = 100; //samples/sec
            const int NumScans = 3;
            
            DaqError response;

            Console.WriteLine("Standard method test.");

            //Connect - establish USB connection
            response = device.Connect();
            if (response.IsError)
            {
                return response;
            }
            
            //Set Scan Group
            response = device.SetScanGroup(scanGroup);
            if (response.IsError)
            {
                return response;
            }

            //Set Scan Rate
            response = device.SetScanRate(ScanRate);
            if (response.IsError)
            {
                return response;
            }

            //Set Start Trigger
            Trigger startTrigger = Trigger.CreateSoftwareManual(); //start immediately after Arm() is called
            response = device.SetStartTrigger(startTrigger);
            if (response.IsError)
            {
                return response;
            }
            
            //Set Stop Trigger
            Trigger stopTrigger = Trigger.CreateSoftwareManual(); //call Disarm() to stop acquisition
            response = device.SetStopTrigger(stopTrigger);
            if (response.IsError)
            {
                return response;
            }
            
            //Arm
            StreamingFifo fifo;
            response = device.Arm(out fifo);
            if (response.IsError)
            {
                return response;
            }

            //Receive and print data
            int scansReceived = 0;
            while (scansReceived < NumScans)
            {
                if (fifo.Count > 0)
                {
                    double[] scan = fifo.Dequeue();
                    Console.WriteLine("Scan #{0}", scansReceived);
                    for (int i = 0; i < scan.Length; i++)
                    {
                        Console.WriteLine("Result #{0}: {1}", i, scan[i]); 
                    }
                    scansReceived++;
                }
            }
            
            //Disarm - stops data acquisition
            response = device.Disarm();
            if (response.IsError)
            {
                return response;
            }
            
            //Disconnect - disconnect USB connection
            response = device.Disconnect();
            if (response.IsError)
            {
                return response;
            }

            return response;
        }
    }
}
