Imports System
Imports System.Collections.Generic
Imports System.Text
Imports Omega.Daq
Imports Omega.Daq.Daq2401

Namespace VBDotNetSample
    ''' <summary>
    ''' Sample code to demonstrate the use of the Omega.Daq API.  To run the hardware portion of this sample code, connect a 0 - 10 V signal to at least one of channels A1H, A2H, A3H, and A4H.
    ''' </summary>
    Class Program
        Public Shared Sub Main(ByVal args As String())
            'run a test with the demo device
            Console.WriteLine("Testing demo device.")
            Dim demoDevice As Device = New Daq2401Demo("DEMO")
            RunTests(demoDevice)

            'check for real devices
            Dim devices As IList(Of Device) = Device.FindConnectedDevices()
            If devices.Count > 0 Then
                'use the first detected device for testing purposes
                RunTests(devices(0))
            Else
                Console.WriteLine("No devices detected.")
            End If

            Console.Write("Done. Press <Enter> to exit.")
            Console.ReadLine() 'wait for Enter key
        End Sub

        ''' <summary>
        ''' Run test/demonstration methods.
        ''' </summary>
        ''' <param name="device">The device to test.</param>
        Private Shared Sub RunTests(ByVal device As Device)
            Const NumChannels As Integer = 4

            Console.WriteLine("Running tests on device {0}", device.SerialNumber)

            'Create a Scan Group with some Channel objects to be used for the tests
            'use a ChannelConfiguration structure to easily copy the same setup to multiple channels
            Dim configuration As New ChannelConfiguration()
            configuration.InputRange = InputRange.VoltsPlusMinus10
            configuration.ChannelMode = ChannelMode.SingleEndedHigh
            'create channels - we'll be using the High input of each channel, with the range set to +/- 10V
            Dim scanGroup As Channel() = New Channel(NumChannels - 1) {}
            'could also use List<Channel>, or any other ICollection<Channel>
            For i As Integer = 0 To NumChannels - 1
                scanGroup(i) = New Channel(i + 1, configuration)
            Next

            'Demonstrate EasyRead single-step command:
            EasyReadTest(device, scanGroup)
            Console.WriteLine()

            'Demonstrate EasyScan single-step command
            EasyScanTest(device, scanGroup)
            Console.WriteLine()

            'Demonstrate standard method
            Dim standardTest As DaqError = StandardScanTest(device, scanGroup)
            If standardTest.IsError Then
                Console.WriteLine("Error - Code: {0} - Message: {1}", standardTest.Name, standardTest.Message)
                If device.IsConnected Then
                    device.Disconnect()
                End If
            End If
            Console.WriteLine()

            Console.WriteLine()
        End Sub

        ''' <summary>
        ''' Demonstrates the EasyRead single-step command. EasyRead reads one channel at a time.
        ''' </summary>
        ''' <param name="device">The device.</param>
        ''' <param name="scanGroup">The scan group.</param>
        Private Shared Sub EasyReadTest(ByVal device As Device, ByVal scanGroup As ICollection(Of Channel))
            Console.WriteLine("EasyRead single-step command:")

            'call EasyRead on each channel in our scan group array and print the output
            For Each chan As Channel In scanGroup
                Console.Write("Channel {0}: ", chan.Number)
                Dim voltage As Double = 0
                Dim response As DaqError = device.EasyRead(chan, voltage)
                If response.IsError Then
                    Console.WriteLine("Error.")
                Else 'successful read
                    Console.WriteLine(voltage)
                End If
            Next
        End Sub

        ''' <summary>
        ''' Demonstrates the EasyScan single-step command. EasyScan reads multiple channels (the scan group) a specified number of times.
        ''' </summary>
        ''' <param name="device">The device.</param>
        ''' <param name="scanGroup">The scan group.</param>
        Private Shared Sub EasyScanTest(ByVal device As Device, ByVal scanGroup As Channel())
            Const ScanRate As Double = 100 'samples/second
            Const NumScans As Integer = 3
            'the data is returned in a standard one-dimensional array: first one scan, then the next, etc.
            Dim data As Double() = New Double(scanGroup.Length * NumScans - 1) {}
            Console.WriteLine("EasyScan single-step command:")

            'call EasyScan and print the output
            Dim response As DaqError = device.EasyScan(scanGroup, NumScans, ScanRate, data)
            If response.IsError Then
                Console.WriteLine("Error.")
            Else 'successful scan
                'iterate over the results
                For s As Integer = 0 To NumScans - 1
                    Console.WriteLine("Scan #{0}", s)
                    For c As Integer = 0 To scanGroup.Length - 1
                        Dim currentIndex As Integer = (s * scanGroup.Length) + c
                        Console.WriteLine("Result #{0}: {1}", c, data(currentIndex))
                    Next
                Next
            End If
        End Sub

        ''' <summary>
        ''' Demonstrates the standard API for reading data.
        ''' </summary>
        ''' <param name="device">The device.</param>
        ''' <param name="scanGroup">The scan group.</param>
        Private Shared Function StandardScanTest(ByVal device As Device, ByVal scanGroup As Channel()) As DaqError
            Const ScanRate As Double = 100 'samples/sec
            Const NumScans As Integer = 3

            Dim response As DaqError

            Console.WriteLine("Standard method test.")

            'Connect - establish USB connection
            response = device.Connect()
            If response.IsError Then
                Return response
            End If

            'Set Scan Group
            response = device.SetScanGroup(scanGroup)
            If response.IsError Then
                Return response
            End If

            'Set Scan Rate
            response = device.SetScanRate(ScanRate)
            If response.IsError Then
                Return response
            End If

            'Set Start Trigger
            Dim startTrigger As Trigger = Trigger.CreateSoftwareManual()
            'start immediately after Arm() is called
            response = device.SetStartTrigger(startTrigger)
            If response.IsError Then
                Return response
            End If

            'Set Stop Trigger
            Dim stopTrigger As Trigger = Trigger.CreateSoftwareManual() 'call Disarm() to stop acquisition
            response = device.SetStopTrigger(stopTrigger)
            If response.IsError Then
                Return response
            End If

            'Arm
            Dim fifo As StreamingFifo = Nothing
            response = device.Arm(fifo)
            If response.IsError Then
                Return response
            End If

            'Receive and print data
            Dim scansReceived As Integer = 0
            While scansReceived < NumScans
                If fifo.Count > 0 Then
                    Dim scan As Double() = fifo.Dequeue()
                    Console.WriteLine("Scan #{0}", scansReceived)
                    For i As Integer = 0 To scan.Length - 1
                        Console.WriteLine("Result #{0}: {1}", i, scan(i))
                    Next
                    scansReceived += 1
                End If
            End While

            'Disarm - stops data acquisition
            response = device.Disarm()
            If response.IsError Then
                Return response
            End If

            'Disconnect - disconnect USB connection
            response = device.Disconnect()
            If response.IsError Then
                Return response
            End If

            Return response
        End Function
    End Class
End Namespace

